@@ -227,6 +227,43 @@ check_gl (GError **error)
         return g_spawn_check_exit_status (status, error);
 }
 
+static inline void
+maybe_push_env_var (const char* name)
+{
+        const char *value = g_getenv (name);
+
+        if (value)
+                gsm_util_setenv (name, value);
+}
+
+static void
+initialize_gio (void)
+{
+        char *disable_fuse = NULL;
+        char *use_vfs = NULL;
+
+        disable_fuse = g_strdup (g_getenv ("GVFS_DISABLE_FUSE"));
+        use_vfs = g_strdup (g_getenv ("GVFS_USE_VFS"));
+
+        g_setenv ("GVFS_DISABLE_FUSE", "1", TRUE);
+        g_setenv ("GIO_USE_VFS", "local", TRUE);
+        g_vfs_get_default ();
+
+        if (use_vfs) {
+                g_setenv ("GIO_USE_VFS", use_vfs, TRUE);
+                g_free (use_vfs);
+        } else {
+                g_unsetenv ("GIO_USE_VFS");
+        }
+
+        if (disable_fuse) {
+                g_setenv ("GIO_DISABLE_FUSE", use_vfs, TRUE);
+                g_free (disable_fuse);
+        } else {
+                g_unsetenv ("GVFS_DISABLE_FUSE");
+        }
+}
+
 int
 main (int argc, char **argv)
 {
@@ -254,6 +291,9 @@ main (int argc, char **argv)
                 gsm_util_init_error (TRUE, "%s", error->message);
         }
 
+        /* Make sure we initialize gio in a way that does not autostart any daemon */
+        initialize_gio ();
+
         bindtextdomain (GETTEXT_PACKAGE, LOCALE_DIR);
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
         textdomain (GETTEXT_PACKAGE);
@@ -340,6 +380,26 @@ main (int argc, char **argv)
         if (g_getenv ("XDG_CURRENT_DESKTOP") == NULL)
             gsm_util_setenv ("XDG_CURRENT_DESKTOP", "GNOME");
 
+        /* Push locale variables to dbus-daemon */
+        maybe_push_env_var ("LC_TIME");
+        maybe_push_env_var ("LC_NUMERIC");
+        maybe_push_env_var ("LC_MONETARY");
+        maybe_push_env_var ("LC_MEASUREMENT");
+        maybe_push_env_var ("LC_PAPER");
+
+        {
+                gchar *ibus_path;
+
+                ibus_path = g_find_program_in_path("ibus-daemon");
+
+                if (ibus_path) {
+                        gsm_util_setenv ("QT_IM_MODULE", "ibus");
+                        gsm_util_setenv ("XMODIFIERS", "@im=ibus");
+                }
+
+                g_free (ibus_path);
+        }
+
         /* Some third-party programs rely on GNOME_DESKTOP_SESSION_ID to
          * detect if GNOME is running. We keep this for compatibility reasons.
          */
@@ -349,6 +409,18 @@ main (int argc, char **argv)
          */
         gsm_util_setenv ("XDG_MENU_PREFIX", "gnome-");
 
+        /* hack to fix keyring until we can reorder things in 3.20
+         * https://bugzilla.gnome.org/show_bug.cgi?id=738205
+         */
+        if (g_strcmp0 (g_getenv ("XDG_SESSION_TYPE"), "wayland") == 0 &&
+            g_getenv ("GSM_SKIP_SSH_AGENT_WORKAROUND") == NULL) {
+                char *ssh_socket;
+
+                ssh_socket = g_build_filename (g_get_user_runtime_dir (), "keyring", "ssh", NULL);
+                gsm_util_setenv ("SSH_AUTH_SOCK", ssh_socket);
+                g_free (ssh_socket);
+        }
+
         gsm_util_set_autostart_dirs (override_autostart_dirs);
         session_name = opt_session_name;
 
